import http from 'k6/http'
import { group, sleep, check } from 'k6'

import {
  sdgsQuery,
  topicsQuery,
  skillsQuery,
  discoverSpacesQuery,
  spacesQuery,
  spaceByIdQuery,
  spacePostsBySpaceIdQuery,
  mySpacesQuery,
} from './queries.js'
import { authenticatedUserQuery } from '../common/queries.js'
import {
  createSpacePostMutation,
  updateSpacePostMutation,
  deleteSpacePostMutation,
  createPostCommentMutation,
  updatePostReactionMutation,
  updatePostCommentReactionMutation,
  deletePostCommentMutation,
} from '../posts/mutations.js'
import { headers, websiteUrl } from '../const.js'
import { addBatchRequest, checkResponse, createPostRequest, isResponseValid } from '../utils.js'
import { checkFailureRate, timeToFirstByte } from '../main.js'

export function spaces(data) {
  group('Load space page', function () {
    const res = http.get(websiteUrl + '/spaces', {
      headers,
    })

    const checkRes = check(res, {
      'is status 200': (r) => r.status === 200,
    })

    // Record check failures
    checkFailureRate.add(!checkRes)

    // Record time to first byte and tag it with the URL to be able to filter the results in Insights
    timeToFirstByte.add(res.timings.waiting, { ttfbURL: res.url })

    group('Unauthenticated user', function () {
      // SDGs, Topics and Skills
      const responses = http.batch([
        addBatchRequest(sdgsQuery, null, { name: 'SdgsQuery' }),
        addBatchRequest(topicsQuery, null, { name: 'TopicsQuery' }),
        addBatchRequest(skillsQuery, null, { name: 'SkillsQuery' }),
      ])

      checkResponse(responses[0])
      checkResponse(responses[1])
      checkResponse(responses[2])
      sleep(Math.random())

      // Discover spaces
      if (isResponseValid(responses[2])) {
        const skills = JSON.parse(responses[2].body).data.skills.data
        for (let i = 0; i < skills.length; i++) {
          const resDiscoverSpaces = createPostRequest(
            discoverSpacesQuery,
            {
              topicIds: [],
              skillIds: [skills[i].id],
              offset: 0,
              limit: 5,
            },
            { name: 'DiscoverSpacesQuery' },
          )
          checkResponse(resDiscoverSpaces)
          sleep(Math.random())
        }
      }

      if (isResponseValid(responses[1])) {
        const topics = JSON.parse(responses[1].body).data.topics.data
        for (let i = 0; i < topics.length; i++) {
          const resDiscoverSpaces = createPostRequest(
            discoverSpacesQuery,
            {
              topicIds: [topics[i].id],
              skillIds: [],
              offset: 0,
              limit: 5,
            },
            { name: 'DiscoverSpacesQuery' },
          )
          checkResponse(resDiscoverSpaces)
          sleep(0.1)
        }

        // Spaces
        const resSpaces = createPostRequest(
          spacesQuery,
          {
            topicSlug: topics[0].slug,
            skillSlugs: [],
            offset: 0,
            limit: 12,
          },
          { name: 'Spaces' },
        )

        checkResponse(resSpaces)
        sleep(0.1)

        // Space details
        if (isResponseValid(resSpaces)) {
          const spaceIdOrName = JSON.parse(resSpaces.body).data.spaces.data[0].name
          const responses = http.batch([
            addBatchRequest(
              spaceByIdQuery,
              {
                idOrName: spaceIdOrName,
              },
              { name: 'SpaceById' },
            ),
            addBatchRequest(
              spacePostsBySpaceIdQuery,
              {
                idOrName: spaceIdOrName,
                offset: 0,
                limit: 10,
                visibility: 'ALL',
              },
              { name: 'SpacePostsBySpaceId' },
            ),
          ])

          checkResponse(responses[0])
          checkResponse(responses[1])
          sleep(Math.random())
        }
      }
    })

    group('Authenticated user', function () {
      // SDGs, Topics and Skills
      const responses = http.batch([
        addBatchRequest(sdgsQuery, null, { name: 'SdgsQuery' }, data.graphQLAuthenticatedUserHeaders),
        addBatchRequest(topicsQuery, null, { name: 'TopicsQuery' }, data.graphQLAuthenticatedUserHeaders),
        addBatchRequest(skillsQuery, null, { name: 'SkillsQuery' }, data.graphQLAuthenticatedUserHeaders),
        addBatchRequest(
          authenticatedUserQuery,
          null,
          { name: 'AuthenticatedUserQuery' },
          data.graphQLAuthenticatedUserHeaders,
        ),
        addBatchRequest(mySpacesQuery, null, { name: 'MySpacesQuery' }, data.graphQLAuthenticatedUserHeaders),
      ])

      checkResponse(responses[0])
      checkResponse(responses[1])
      checkResponse(responses[2])
      checkResponse(responses[3])
      checkResponse(responses[4])
      sleep(Math.random())

      // Discover spaces
      if (isResponseValid(responses[2])) {
        const skills = JSON.parse(responses[2].body).data.skills.data
        for (let i = 0; i < skills.length; i++) {
          const resDiscoverSpaces = createPostRequest(
            discoverSpacesQuery,
            {
              topicIds: [],
              skillIds: [skills[i].id],
              offset: 0,
              limit: 5,
            },
            { name: 'DiscoverSpacesQuery' },
            data.graphQLAuthenticatedUserHeaders,
          )
          checkResponse(resDiscoverSpaces)
          sleep(0.1)
        }
      }

      if (isResponseValid(responses[1])) {
        const topics = JSON.parse(responses[1].body).data.topics.data
        for (let i = 0; i < topics.length; i++) {
          const resDiscoverSpaces = createPostRequest(
            discoverSpacesQuery,
            {
              topicIds: [topics[i].id],
              skillIds: [],
              offset: 0,
              limit: 5,
            },
            { name: 'DiscoverSpacesQuery' },
            data.graphQLAuthenticatedUserHeaders,
          )
          checkResponse(resDiscoverSpaces)
          sleep(0.1)
        }

        // Spaces
        const resSpaces = createPostRequest(
          spacesQuery,
          {
            topicSlug: topics[0].slug,
            skillSlugs: [],
            offset: 0,
            limit: 12,
          },
          { name: 'Spaces' },
          data.graphQLAuthenticatedUserHeaders,
        )

        checkResponse(resSpaces)
        sleep(Math.random())

        // Space details
        if (isResponseValid(resSpaces)) {
          const spaceIdOrName = JSON.parse(resSpaces.body).data.spaces.data[0].name
          const responses = http.batch([
            addBatchRequest(
              spaceByIdQuery,
              {
                idOrName: spaceIdOrName,
              },
              { name: 'SpaceById' },
              data.graphQLAuthenticatedUserHeaders,
            ),
            addBatchRequest(
              spacePostsBySpaceIdQuery,
              {
                idOrName: spaceIdOrName,
                offset: 0,
                limit: 10,
                visibility: 'ALL',
              },
              { name: 'SpacePostsBySpaceId' },
              data.graphQLAuthenticatedUserHeaders,
            ),
          ])

          checkResponse(responses[0])
          checkResponse(responses[1])
          sleep(Math.random())
        }
      }

      // Space/Post Mutations (not in production environment)
      if (isResponseValid(responses[4]) && __ENV.PRODUCTION !== '1') {
        const mySpaces = JSON.parse(responses[4].body).data.mySpaces
        if (mySpaces[0].spaces.totalResults > 0) {
          // Create space post mutation
          const spaceId = mySpaces[0].spaces.data[0].id

          const responseCreatePostMutation = createPostRequest(
            createSpacePostMutation,
            {
              input: {
                spaceId,
                title: 'test',
                text: 'test',
                visibility: 'MEMBERS',
              },
            },
            { name: 'CreateSpacePost' },
            data.graphQLAuthenticatedUserHeaders,
          )

          checkResponse(responseCreatePostMutation)
          sleep(0.1)

          if (isResponseValid(responseCreatePostMutation)) {
            const postId = JSON.parse(responseCreatePostMutation.body).data.createSpacePost2.id

            // Update space post mutation
            const responseUpdatePostMutation = createPostRequest(
              updateSpacePostMutation,
              {
                input: {
                  postId,
                  title: 'test updated',
                  text: 'test updated',
                  visibility: 'PUBLIC',
                },
              },
              { name: 'UpdateSpacePost' },
              data.graphQLAuthenticatedUserHeaders,
            )

            checkResponse(responseUpdatePostMutation)
            sleep(0.1)

            // Update post reaction mutation
            const responseUpdatePostReactionMutation = createPostRequest(
              updatePostReactionMutation,
              {
                id: postId,
                reactionType: 'HEART',
                reactionAction: 'SET',
              },
              { name: 'UpdatePostReaction' },
              data.graphQLAuthenticatedUserHeaders,
            )

            checkResponse(responseUpdatePostReactionMutation)
            sleep(0.1)

            // Create post comment mutation
            const responseCreatePostCommentMutation = createPostRequest(
              createPostCommentMutation,
              {
                input: {
                  postId,
                  text: 'test comment',
                },
              },
              { name: 'CreatePostComment' },
              data.graphQLAuthenticatedUserHeaders,
            )

            checkResponse(responseCreatePostCommentMutation)
            sleep(0.1)

            if (isResponseValid(responseCreatePostCommentMutation)) {
              const commentId = JSON.parse(responseCreatePostCommentMutation.body).data.updatePostComment.postComment.id

              // Update post comment reaction mutation
              const responseUpdatePostCommentReactionMutation = createPostRequest(
                updatePostCommentReactionMutation,
                {
                  id: commentId,
                  reactionType: 'HEART',
                  reactionAction: 'SET',
                },
                { name: 'UpdatePostCommentReaction' },
                data.graphQLAuthenticatedUserHeaders,
              )

              checkResponse(responseUpdatePostCommentReactionMutation)
              sleep(0.1)

              // Delete post comment mutation
              const responseDeletePostCommentMutation = createPostRequest(
                deletePostCommentMutation,
                {
                  id: commentId,
                },
                { name: 'DeletePostComment' },
                data.graphQLAuthenticatedUserHeaders,
              )

              checkResponse(responseDeletePostCommentMutation)
              sleep(0.1)
            }

            // Delete space post mutation
            const responseDeletePostMutation = createPostRequest(
              deleteSpacePostMutation,
              {
                id: postId,
              },
              { name: 'DeleteSpacePost' },
              data.graphQLAuthenticatedUserHeaders,
            )

            checkResponse(responseDeletePostMutation)
            sleep(0.1)
          }
        }
      }
    })
  })
}
